# -*- coding: utf-8 -*-
"""
Created on Fri Aug 25 11:27:54 2023

@author: msbee
"""

import numpy as np
from scipy import fftpack
import matplotlib.pyplot as plt
from scipy.stats import cauchy, trapezoid, norm
import time

plt.close('all')
start = time.time()

v_0 = 3.843 * 10**10    #laser and resonance frequency


samples = 10**6   #How many time points are needed i,e., Sampling Frequency

# define time variable
ti = -1 * 10**-6
tf = 1 * 10**-6
sample_duration = tf-ti
sample_interval = sample_duration / samples;
t = np.linspace(ti, tf, samples)

print('min frequency = ', '{:.2E}'.format(1/sample_duration))
print('max frequency = ', '{:.2E}'.format(1/sample_interval), '\n')
if 1/sample_interval < 2*v_0:
    print('definitely not enough samples \n'.upper())

#%% define pulse characteristics, trapezoid or gaussian shape
c = 0.0002  #pulse rising scale
d = 1 - c     #pulse falling scale
pulse = trapezoid.pdf(t, c, d, loc = 0, scale = 100 * 10**-6)
#pulse = 1 - np.e**(-t / (9 * 10**-9))

#set pulse equal to 0 before certain time
indicies = np.where((t < 0))
pulse[indicies] = 0
#add oscillation and remove DC offset
pulse = pulse * (np.sin(2*np.pi*v_0*t)**2)
pulse = pulse / np.max(pulse)
mean = np.mean(pulse)
pulse = pulse - mean


#%% Frequency domain representation and filtering
frequencies = np.arange(int(samples / 2)) / sample_duration
fft = fftpack.fft(pulse)
fft = fft[range(int(len(pulse) / 2))] # Exclude sampling frequency
fft_filtered = fft.copy()
filter_shape = norm.pdf(frequencies, loc = 2*v_0,  scale = 6.0746 * 10**6)
#filter_shape = cauchy.pdf(frequencies, 2*v_0, 6.0746 * 10**6)
filter_shape = 1 * filter_shape / np.max(filter_shape)

absorb = True  #apply filter
if absorb:
    for i, value in enumerate(fft_filtered):
        fft_filtered[i] = 1 * value * (filter_shape[i])
        
y_inv = fftpack.ifft((fft_filtered))
y_inv = np.conjugate(y_inv) * y_inv
y_inv = y_inv.real  #should be all real, but change data type to only real values
y_inv = y_inv / np.max(y_inv)
t_inv = np.linspace(ti, tf, len(y_inv))


#%% plotting
plots = [True, True, True, True]

if plots[0]:
    plt.figure(0)
    plt.plot(t[::10] * 10**9, pulse[::10] + mean, 'b', label = 'Pulse')
    plt.plot(t_inv[::10] * 10**9, y_inv[::10], 'r', label = 'Calculated OD')
    plt.xlim([-100, 100])
    plt.legend(loc = 'best')
    plt.title('Time Domain Signals')
    plt.xlabel('Time (ns)')

if plots[1]:
    plt.figure(1)
    fft = fft / np.max(abs(fft)) #normalize
    plt.semilogx(frequencies, abs(fft), 'b')
    plt.xlabel("Frequency (Hz)")
    plt.title('Original Spectrum')

if plots[2]:
    plt.figure(2)
    fft_filtered = fft_filtered / np.max(abs(fft_filtered))  #normalize
    #plt.plot(frequencies[::10], abs(fft_filtered)[::10], 'r')
    indices = np.where((frequencies >= 2*v_0 - 10**7) & (frequencies <= 2*v_0 + 10**7))
    plt.plot((frequencies[indices] - (2*v_0)) / (10**6), abs(fft_filtered)[indices], 'r-o', label='filtered')
    plt.title('Filtered Spectrum')
    plt.xlabel("V_0 detuning (MHz)")

if plots[3]: 
    plt.figure(3)
    plt.plot(frequencies[::10], filter_shape[::10], 'b-o')
    plt.title('Filter Shape')
    plt.xlabel("Frequency (Hz)")
    plt.ylabel("Absorption")
    plt.xlim([2*v_0 - 10**8, 2*v_0 + 10**8])

#%%

def calculate_rise_time(time, data, plot):
    max_value = np.max(data)
    min_value = np.min(data)
    indicies = np.where((time >= -500 * 10**-9) & (time <= 500 * 10**-9))  #rising edge will be somewhere in here
    data = data[indicies]
    time = time[indicies]
    
    if plot:
        plt.figure(4)
        plt.plot(time*10**9, data, 'b')
        plt.title('filtered signal rise-time zoom')
        plt.xlabel('Time (ns)')
        
    #define start and end values to look for a 90-10 rise time calculation
    start_value = (max_value - min_value) * 0.1
    end_value = (max_value - min_value) * 0.9

    # Find the first index where data is greater than or equal to start_value
    start_index = next((i for i, value in enumerate(data) if value >= start_value), None)
    # Find the last index where data is less than or equal to end_value
    end_index = next((i for i, value in reversed(list(enumerate(data))) if value <= end_value), None)
    if start_index is not None and end_index is not None:
        rise_time = time[end_index] - time[start_index]
        return rise_time
    else:
        return None
#%%

rise_time = calculate_rise_time(t_inv, y_inv, True)
print("90-10 Rise Time:", '{0:.2f}'.format(rise_time * 10**9), "nanoseconds")
print("Time Constant:", '{0:.2f}'.format(rise_time * 10**9 / 2.197), "nanoseconds \n")  #conversion factor for gaussian rise time
print('run time = ', '{0:.2f}'.format(time.time() - start), 'seconds')
