# -*- coding: utf-8 -*-
"""
Created on Fri Jun 23 13:23:24 2023

@author: Matt Beede
"""

import numpy as np
import matplotlib.pyplot as plt
from scipy.fft import fft, fftfreq, rfft, rfftfreq, irfft, ifft
from scipy import signal



plt.close('all')

#26.2 ns half life (time constant)

order = 9
sample = 1*10**2
time = 10**-(order+3) #in Seconds
N = int(sample*2*time)
sampling_rate = (sample / time)

offset = 2*10**(-order-5)
width = .5*10**(-order-5)

sigma = 10/2.2*10**-7.5


def lightWave(E0, lamda, freq):
    t = np.linspace(-time, time, sample)
    
    
    omega = 2 * np.pi * freq 
    dt = t[2]-t[1]
    E = E0 * np.cos( omega * t)
    # E = E - np.mean(E)
    return t, E, dt
 
def Intensity(E):
    I = (E*np.conj(E))
    # I = E**2
    DCoffset=np.mean(I)
    I = I - DCoffset
    # print(np.min(I), np.mean(I), np.max(I))
    # print(np.min(E), np.mean(E), np.max(E))
    return I , DCoffset


def butter_bandstop_filter(data, lowcut, highcut, fs, order):
    nyq = 0.5 * fs
    low = lowcut / nyq
    high = highcut / nyq

    i, u = signal.butter(order, [low, high], btype='bandstop', analog=True)
    y = signal.lfilter(i, u, data)
    xfilter, yfilter = signal.freqs(i, u)
    return y, xfilter, yfilter

def Normalise(data, DCoffset):
    max = np.max(data)
    min = np.min(data)
    delta = (max-min)
    offset = ((DCoffset - min) / delta)/2
    normalised_data = (data - min) / delta - offset
    
    return normalised_data
    
    
#############################
####Initialise Light Wave####
#############################
lamda = 780 *(10**-order)
freq = 299792458 / lamda #384349305128205.1 (384 THz)
period = 1/freq  #2.601799942545586e-15 (2.6 fs)
omega = 2 * np.pi * freq #2414937906806222.0 (3.4Prad/sec)
x, E, spacing = lightWave(3, lamda, freq)



############
####Pulse###
############

# pulse = 2*np.e**(-(((x-offset)**2)/(2*(width)**2))**4)



y, DCoffset = Intensity(E)#*pulse)
Norm = Normalise(y,DCoffset)
# print(spacing)   2.0000200002047383e-17 (20 as)



print("Wave Generated")







############################################
####Fourier Transform to frequency space####
############################################
yf = fft(Norm)

# xf = rfftfreq(2*len(x)-1, spacing/2)
xf = fftfreq(len(x),spacing)
# print(xf)
# print(len(x))
# print(len(xf))

print("FFT Complete")





###########################
####Filter by Frequency####
###########################
reduction = .5
lowcut = np.floor(2*freq-10**13)
highcut = np.ceil(2*freq+10**13)
yfilter = 0 * abs(yf)



# print(len(xf))
# print(len(yf))





# yf2, xfilter, yfilter=  butter_bandstop_filter(yf, lowcut, highcut, sample, 1)

b, a = signal.butter(1, [lowcut, highcut], "bandstop", analog = True)
xfilter, yfilter = signal.freqs(b, a, len(yf))
final = signal.filtfilt(b, a, ~np.isnan(Norm))


# sos = signal.ellip(1, 50, 40,[lowcut, highcut], "bandstop", analog = True, output='sos')
# xfilter, yfilter = signal.sosfreqz(sos, len(yf))
# final = signal.sosfiltfilt(sos, Norm)


# final = signal.filtfilt(b, a, Norm)

# print(yf[1530])
# print(final[1530])


# for i, ele in enumerate(xf):
#     if ele < highcut and ele > lowcut:
#         print(i)









print("Filter Complete")



#########################################################
####Inverse Fourier Transform to return to time space####
#########################################################
# final = ifft(yf2)
# final=final#-np.min(final)

print("Inverse FFT Complete")




#################################
####Finding the Optical Depth####
#################################
# OD =  np.log(y/final)

print("Optical Depth Complete")






##############
####Graphs####
##############


xscale = 4*10**-(order+5)
yscale = 10
xfscale = 9*10**(order+5)
yfscale = sample



# fig, ax = plt.subplots(6)
# fig.tight_layout(pad=2)
# ax[0].plot(x,y+DCoffset)
# ax[0].plot(x,final+((1-reduction)*DCoffset))
# ax[0].set_xlim(0,xscale)
# ax[0].set_ylim(-5,yscale)
# ax[0].set_title('Original')



# ax[1].set_title('FFT')
# ax[1].plot(xf, abs(yf))
# ax[1].set_xlim(0,xfscale)
# ax[1].set_ylim(0,yfscale)
# # ax[1].plot(x,y-final)
# # ax[1].plot(x,pulse)
# # ax[1].set_xlim(0,xscale)
# # ax[1].set_ylim(-1,yscale)



# ax[2].plot(abs(xf), abs(yf2))
# ax[2].set_title('Post Filter')
# ax[2].set_xlim(0,xfscale)
# ax[2].set_ylim(0,yfscale)



# ax[3].plot(x, final+(1-reduction)*DCoffset)
# ax[3].set_xlim(0,xscale)
# ax[3].set_title('Inverse FFT')



# ax[4].plot(x,OD)
# ax[4].plot(x,y/final)
# ax[4].set_title('Optical Depth')
# ax[4].set_ylim(-1,1)
# ax[4].set_xlim(0,xscale)



# ax[5].set_title('Filter')
# ax[5].plot(xf, yfilter)
# ax[5].set_xlim(-xfscale,xfscale)



#####################
####Single Graphs####
#####################

# plt.figure(0)
#plt.title("Original Wave Intensity")
# plt.plot(x,y)

# plt.xlim(0,xscale)

# plt.figure(1)
# plt.title("FFT")
# plt.plot(abs(xf), abs(yf))
# plt.xlim(0, xfscale)

# plt.figure(2)
# plt.title("Filtered FFT")
# plt.plot(abs(xf), abs(yf))
# plt.plot(abs(xf), z)
# plt.plot(abs(xf), abs(yf2))
# plt.xlim(0, xfscale)

plt.figure(3)
plt.title("Filtered Wave Intensity (Post IFFT)")
plt.plot(x,final)

# plt.figure(4)
#plt.title("Optical Depth")
# plt.plot(x,OD)
# plt.plot(x,y/final)
# plt.ylim(-1,1)
# plt.xlim(0,xscale)
# plt.figure(5)
# plt.plot(xf, yfilter)
# plt.ylim(0,xfscale)

# plt.figure(6)
#plt.title("Pulse")
# plt.plot(x, pulse)
# plt.plot(x, .1*y)
# plt.xlim(0,xscale)

plt.figure(7)
plt.title("Filter")
plt.plot(xfilter, yfilter)
plt.xlim(0,xfscale)

# plt.figure(8)
# plt.title("Normalised Intensity")
# plt.plot(x,Norm)
# plt.xlim(0,xscale)

# plt.figure(9)
# plt.plot(xf, z)



########################
####Diagnostic stuff####
########################

# for i, ele in enumerate(y,100):
#     if i<111:
#         print(y[i])
#     else:
        # pass
    
# for i, ele in enumerate(yf):
#     if i<11:
#         print(yf[i])
#     else:
#         pass
# for i, ele in enumerate(yf2):
#     if i<11:
#         print(yf2[i])
#     else:
#         pass
    
# for i, ele in enumerate(yf):
#     if i<11:
#         print(yf2[i]/yf[i])
#     else:
#         pass
    
# for i, ele in enumerate(yfilter,10):
#     if i<101:
#         print(yf2[i]/yf[i])
#     else:
#         pass

# for i, ele in enumerate(xf,1000):
#     if i<1010:
#         print(xf[i])
#     else:
#         pass


# print(len(x))
# print(len(y))
# print(len(xf))
# print(len(yf))
# print(len(yf2))
# print(len(final))
# print(len(OD))

# print(x)
# print(final)


# print(type(yfilter[0]))
# print(type(yf[0]))
# print(type(yf2[0]))