from eink import *
from math import floor
from machine import Pin, SPI
from utime import ticks_ms, ticks_diff, sleep
import sdcard
import uos
from machine import freq
freq(200*10**6)


# Assign chip select (CS) pin (and start it high)
cs = machine.Pin(1, machine.Pin.OUT)

# Intialize SPI peripheral (start with 1 MHz)
spi = machine.SPI(0,
                  baudrate=1000000,
                  polarity=0,
                  phase=0,
                  bits=8,
                  firstbit=machine.SPI.MSB,
                  sck=machine.Pin(2),
                  mosi=machine.Pin(3),
                  miso=machine.Pin(4))



try:
    # Initialize SD card
    sd = sdcard.SDCard(spi, cs)

    # Mount filesystem
    vfs = uos.VfsFat(sd)
    uos.mount(vfs, "/sd")

except:
    print('no sd card')
    pass

# key_0 = Pin(15, Pin.IN, Pin.PULL_UP)
# key_1 = Pin(17, Pin.IN, Pin.PULL_UP)
# key_2 = Pin(2, Pin.IN, Pin.PULL_UP)

epd = EPD_5in65()


def picture(xstart, ystart, filename,filename2):
    start = ticks_ms()
    epd.EPD_5IN65F_Init() #reinitialize screen
    print('filling buffer with %s' %filename)
    with open(filename,'r') as f:
        for y, line in enumerate(f):
            for x, pixel in enumerate(line.strip()):
                if x + xstart < 601 and y + ystart < 449:
                    try:
                        epd.pixel(x, y, int(pixel))
                    except Exception as e:
                        print(e)
                else:
                    print('Out Of Range')
                    
    print('filling buffer with %s' %filename2)
    with open(filename2,'r') as f:
        for y, line in enumerate(f):
            if y==0:
                offset = floor((448 - int(line))/2)
                print(offset)
            else:
                for x, pixel in enumerate(line.strip()):
                    if x + xstart < 601 and y + ystart < 449:
                        try:
                            epd.pixel(x, y - 1 + offset, int(pixel))
                        except Exception as e:
                            print(e)
                    else:
                        print('Out Of Range')
        
                    
    middle = ticks_ms()
    print('fill time = %s seconds' %str((middle-start)/1000))
    print('rendering buffer')
    epd.render(epd.buffer)
    end = ticks_ms()
    time = end-start
    print('rendered')
    print('total time = %s seconds' %str(time/1000))
    epd.Sleep()  #sleep or otherwise screen will stay in high voltage mode and break

# def view_pics():
#     with open('current_pic.txt','r') as f:
#         i=int(f.read())
#         f.close()
#     filenames=list()
#     for filename in uos.listdir('/sd/pics'):
#         filenames.append('/sd/pics/'+filename)
#     if i not in range(len(filenames)):
#         i = 0
#     while True:
#         picture(filenames[i])
#         i+=1
#         i=i%len(filenames) 
#         with open('current_pic.txt','w') as f:
#             f.write(str(i))
#             f.close()
#         sleep(1000*30)
        
           
    
#view_pics()
def display_screen(sleep_time):   #sleep time in minutes
    pic_directory='/sd/images'
    filenames=list()
    for filename in uos.listdir(pic_directory):
        filenames.append(filename)
    while True:
        for item in filenames:
            picture(0, 0, '/sd/blank_template.txt','/sd/images/'+item)
            sleep(sleep_time*60)       

while True:
    display_screen(120)



